# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.get_snippet_response import GetSnippetResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.snippet import SnippetParams
from ..types.upsert_snippet_response import UpsertSnippetResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.delete_snippet_response import DeleteSnippetResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSnippetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetSnippetResponse]:
        """
        Retrieves your snippet from a Square Online site. A site can contain snippets from multiple snippet applications, but you can retrieve only the snippet that was added by your application.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSnippetResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/sites/{jsonable_encoder(site_id)}/snippet",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSnippetResponse,
                    construct_type(
                        type_=GetSnippetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert(
        self, site_id: str, *, snippet: SnippetParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpsertSnippetResponse]:
        """
        Adds a snippet to a Square Online site or updates the existing snippet on the site.
        The snippet code is appended to the end of the `head` element on every page of the site, except checkout pages. A snippet application can add one snippet to a given site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site where you want to add or update the snippet.

        snippet : SnippetParams
            The snippet for the site.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpsertSnippetResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/sites/{jsonable_encoder(site_id)}/snippet",
            method="POST",
            json={
                "snippet": convert_and_respect_annotation_metadata(
                    object_=snippet, annotation=SnippetParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertSnippetResponse,
                    construct_type(
                        type_=UpsertSnippetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteSnippetResponse]:
        """
        Removes your snippet from a Square Online site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteSnippetResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/sites/{jsonable_encoder(site_id)}/snippet",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteSnippetResponse,
                    construct_type(
                        type_=DeleteSnippetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawSnippetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetSnippetResponse]:
        """
        Retrieves your snippet from a Square Online site. A site can contain snippets from multiple snippet applications, but you can retrieve only the snippet that was added by your application.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSnippetResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/sites/{jsonable_encoder(site_id)}/snippet",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSnippetResponse,
                    construct_type(
                        type_=GetSnippetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert(
        self, site_id: str, *, snippet: SnippetParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpsertSnippetResponse]:
        """
        Adds a snippet to a Square Online site or updates the existing snippet on the site.
        The snippet code is appended to the end of the `head` element on every page of the site, except checkout pages. A snippet application can add one snippet to a given site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site where you want to add or update the snippet.

        snippet : SnippetParams
            The snippet for the site.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpsertSnippetResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/sites/{jsonable_encoder(site_id)}/snippet",
            method="POST",
            json={
                "snippet": convert_and_respect_annotation_metadata(
                    object_=snippet, annotation=SnippetParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertSnippetResponse,
                    construct_type(
                        type_=UpsertSnippetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, site_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteSnippetResponse]:
        """
        Removes your snippet from a Square Online site.

        You can call [ListSites](api-endpoint:Sites-ListSites) to get the IDs of the sites that belong to a seller.


        __Note:__ Square Online APIs are publicly available as part of an early access program. For more information, see [Early access program for Square Online APIs](https://developer.squareup.com/docs/online-api#early-access-program-for-square-online-apis).

        Parameters
        ----------
        site_id : str
            The ID of the site that contains the snippet.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteSnippetResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/sites/{jsonable_encoder(site_id)}/snippet",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteSnippetResponse,
                    construct_type(
                        type_=DeleteSnippetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.invoice_request_method import InvoiceRequestMethod
from ..types.invoice_request_type import InvoiceRequestType
from .money import MoneyParams
from ..types.invoice_automatic_payment_source import InvoiceAutomaticPaymentSource
from .invoice_payment_reminder import InvoicePaymentReminderParams


class InvoicePaymentRequestParams(typing_extensions.TypedDict):
    """
    Represents a payment request for an [invoice](entity:Invoice). Invoices can specify a maximum
    of 13 payment requests, with up to 12 `INSTALLMENT` request types. For more information,
    see [Configuring payment requests](https://developer.squareup.com/docs/invoices-api/create-publish-invoices#payment-requests).

    Adding `INSTALLMENT` payment requests to an invoice requires an
    [Invoices Plus subscription](https://developer.squareup.com/docs/invoices-api/overview#invoices-plus-subscription).
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the payment request in an [invoice](entity:Invoice).
    """

    request_method: typing_extensions.NotRequired[InvoiceRequestMethod]
    """
    Indicates how Square processes the payment request. DEPRECATED at version 2021-01-21. Replaced by the
    `Invoice.delivery_method` and `InvoicePaymentRequest.automatic_payment_source` fields.
    
    One of the following is required when creating an invoice:
    - (Recommended) The `delivery_method` field of the invoice. To configure an automatic payment, the
    `automatic_payment_source` field of the payment request is also required.
    - This `request_method` field. Note that `invoice` objects returned in responses do not include `request_method`.
    See [InvoiceRequestMethod](#type-invoicerequestmethod) for possible values
    """

    request_type: typing_extensions.NotRequired[InvoiceRequestType]
    """
    Identifies the payment request type. This type defines how the payment request amount is determined. 
    This field is required to create a payment request.
    See [InvoiceRequestType](#type-invoicerequesttype) for possible values
    """

    due_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The due date (in the invoice's time zone) for the payment request, in `YYYY-MM-DD` format. This field
    is required to create a payment request. If an `automatic_payment_source` is defined for the request, Square
    charges the payment source on this date.
    
    After this date, the invoice becomes overdue. For example, a payment `due_date` of 2021-03-09 with a `timezone`
    of America/Los\_Angeles becomes overdue at midnight on March 9 in America/Los\_Angeles (which equals a UTC
    timestamp of 2021-03-10T08:00:00Z).
    """

    fixed_amount_requested_money: typing_extensions.NotRequired[MoneyParams]
    """
    If the payment request specifies `DEPOSIT` or `INSTALLMENT` as the `request_type`, 
    this indicates the request amount.
    You cannot specify this when `request_type` is `BALANCE` or when the 
    payment request includes the `percentage_requested` field.
    """

    percentage_requested: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Specifies the amount for the payment request in percentage:
    
    - When the payment `request_type` is `DEPOSIT`, it is the percentage of the order's total amount.
    - When the payment `request_type` is `INSTALLMENT`, it is the percentage of the order's total less 
    the deposit, if requested. The sum of the `percentage_requested` in all installment 
    payment requests must be equal to 100.
    
    You cannot specify this when the payment `request_type` is `BALANCE` or when the 
    payment request specifies the `fixed_amount_requested_money` field.
    """

    tipping_enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If set to true, the Square-hosted invoice page (the `public_url` field of the invoice) 
    provides a place for the customer to pay a tip. 
    
    This field is allowed only on the final payment request  
    and the payment `request_type` must be `BALANCE` or `INSTALLMENT`.
    """

    automatic_payment_source: typing_extensions.NotRequired[InvoiceAutomaticPaymentSource]
    """
    The payment method for an automatic payment.
    
    The default value is `NONE`.
    See [InvoiceAutomaticPaymentSource](#type-invoiceautomaticpaymentsource) for possible values
    """

    card_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the credit or debit card on file to charge for the payment request. To get the cards on file for a customer,
    call [ListCards](api-endpoint:Cards-ListCards) and include the `customer_id` of the invoice recipient.
    """

    reminders: typing_extensions.NotRequired[typing.Optional[typing.Sequence[InvoicePaymentReminderParams]]]
    """
    A list of one or more reminders to send for the payment request.
    """

    computed_amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of the payment request, computed using the order amount and information from the various payment
    request fields (`request_type`, `fixed_amount_requested_money`, and `percentage_requested`).
    """

    total_completed_amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money already paid for the specific payment request. 
    This amount might include a rounding adjustment if the most recent invoice payment 
    was in cash in a currency that rounds cash payments (such as, `CAD` or `AUD`).
    """

    rounding_adjustment_included_money: typing_extensions.NotRequired[MoneyParams]
    """
    If the most recent payment was a cash payment 
    in a currency that rounds cash payments (such as, `CAD` or `AUD`) and the payment 
    is rounded from `computed_amount_money` in the payment request, then this 
    field specifies the rounding adjustment applied. This amount 
    might be negative.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .custom_attribute import CustomAttributeParams
import typing
from .error import ErrorParams


class UpsertBookingCustomAttributeResponseParams(typing_extensions.TypedDict):
    """
    Represents an [UpsertBookingCustomAttribute](api-endpoint:BookingCustomAttributes-UpsertBookingCustomAttribute) response.
    Either `custom_attribute_definition` or `errors` is present in the response.
    """

    custom_attribute: typing_extensions.NotRequired[CustomAttributeParams]
    """
    The new or updated custom attribute.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.gift_card_activity_block_reason import GiftCardActivityBlockReason


class GiftCardActivityBlockParams(typing_extensions.TypedDict):
    """
    Represents details about a `BLOCK` [gift card activity type](entity:GiftCardActivityType).
    """

    reason: GiftCardActivityBlockReason
    """
    The reason the gift card was blocked.
    See [Reason](#type-reason) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.cash_drawer_shift_state import CashDrawerShiftState
import typing
from .money import MoneyParams
from .cash_drawer_device import CashDrawerDeviceParams


class CashDrawerShiftParams(typing_extensions.TypedDict):
    """
    This model gives the details of a cash drawer shift.
    The cash_payment_money, cash_refund_money, cash_paid_in_money,
    and cash_paid_out_money fields are all computed by summing their respective
    event types.
    """

    id: typing_extensions.NotRequired[str]
    """
    The shift unique ID.
    """

    state: typing_extensions.NotRequired[CashDrawerShiftState]
    """
    The shift current state.
    See [CashDrawerShiftState](#type-cashdrawershiftstate) for possible values
    """

    opened_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The time when the shift began, in ISO 8601 format.
    """

    ended_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The time when the shift ended, in ISO 8601 format.
    """

    closed_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The time when the shift was closed, in ISO 8601 format.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The free-form text description of a cash drawer by an employee.
    """

    opened_cash_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money in the cash drawer at the start of the shift.
    The amount must be greater than or equal to zero.
    """

    cash_payment_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money added to the cash drawer from cash payments.
    This is computed by summing all events with the types CASH_TENDER_PAYMENT and
    CASH_TENDER_CANCELED_PAYMENT. The amount is always greater than or equal to
    zero.
    """

    cash_refunds_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money removed from the cash drawer from cash refunds.
    It is computed by summing the events of type CASH_TENDER_REFUND. The amount
    is always greater than or equal to zero.
    """

    cash_paid_in_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money added to the cash drawer for reasons other than cash
    payments. It is computed by summing the events of type PAID_IN. The amount is
    always greater than or equal to zero.
    """

    cash_paid_out_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money removed from the cash drawer for reasons other than
    cash refunds. It is computed by summing the events of type PAID_OUT. The amount
    is always greater than or equal to zero.
    """

    expected_cash_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money that should be in the cash drawer at the end of the
    shift, based on the shift's other money amounts.
    This can be negative if employees have not correctly recorded all the events
    on the cash drawer.
    cash_paid_out_money is a summation of amounts from cash_payment_money (zero
    or positive), cash_refunds_money (zero or negative), cash_paid_in_money (zero
    or positive), and cash_paid_out_money (zero or negative) event types.
    """

    closed_cash_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money found in the cash drawer at the end of the shift
    by an auditing employee. The amount should be positive.
    """

    device: typing_extensions.NotRequired[CashDrawerDeviceParams]
    """
    The device running Square Point of Sale that was connected to the cash drawer.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The shift start time in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The shift updated at time in RFC 3339 format.
    """

    location_id: typing_extensions.NotRequired[str]
    """
    The ID of the location the cash drawer shift belongs to.
    """

    team_member_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The IDs of all team members that were logged into Square Point of Sale at any
    point while the cash drawer shift was open.
    """

    opening_team_member_id: typing_extensions.NotRequired[str]
    """
    The ID of the team member that started the cash drawer shift.
    """

    ending_team_member_id: typing_extensions.NotRequired[str]
    """
    The ID of the team member that ended the cash drawer shift.
    """

    closing_team_member_id: typing_extensions.NotRequired[str]
    """
    The ID of the team member that closed the cash drawer shift by auditing
    the cash drawer contents.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.device_attributes_device_type import DeviceAttributesDeviceType
import typing_extensions
import typing


class DeviceAttributesParams(typing_extensions.TypedDict):
    type: DeviceAttributesDeviceType
    """
    The device type.
    See [DeviceType](#type-devicetype) for possible values
    """

    manufacturer: str
    """
    The maker of the device.
    """

    model: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The specific model of the device.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A seller-specified name for the device.
    """

    manufacturers_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The manufacturer-supplied identifier for the device (where available). In many cases,
    this identifier will be a serial number.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The RFC 3339-formatted value of the most recent update to the device information.
    (Could represent any field update on the device.)
    """

    version: typing_extensions.NotRequired[str]
    """
    The current version of software installed on the device.
    """

    merchant_token: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The merchant_token identifying the merchant controlling the device.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .card import Card
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CreateCardResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body of
    a request to the [CreateCard](api-endpoint:Cards-CreateCard) endpoint.

    Note: if there are errors processing the request, the card field will not be
    present.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Errors resulting from the request.
    """

    card: typing.Optional[Card] = pydantic.Field(default=None)
    """
    The card created by the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogInfoResponseLimits(UncheckedBaseModel):
    batch_upsert_max_objects_per_batch: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of objects that may appear within a single batch in a
    `/v2/catalog/batch-upsert` request.
    """

    batch_upsert_max_total_objects: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of objects that may appear across all batches in a
    `/v2/catalog/batch-upsert` request.
    """

    batch_retrieve_max_object_ids: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of object IDs that may appear in a `/v2/catalog/batch-retrieve`
    request.
    """

    search_max_page_limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of results that may be returned in a page of a
    `/v2/catalog/search` response.
    """

    batch_delete_max_object_ids: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of object IDs that may be included in a single
    `/v2/catalog/batch-delete` request.
    """

    update_item_taxes_max_item_ids: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of item IDs that may be included in a single
    `/v2/catalog/update-item-taxes` request.
    """

    update_item_taxes_max_taxes_to_enable: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of tax IDs to be enabled that may be included in a single
    `/v2/catalog/update-item-taxes` request.
    """

    update_item_taxes_max_taxes_to_disable: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of tax IDs to be disabled that may be included in a single
    `/v2/catalog/update-item-taxes` request.
    """

    update_item_modifier_lists_max_item_ids: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of item IDs that may be included in a single
    `/v2/catalog/update-item-modifier-lists` request.
    """

    update_item_modifier_lists_max_modifier_lists_to_enable: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of modifier list IDs to be enabled that may be included in
    a single `/v2/catalog/update-item-modifier-lists` request.
    """

    update_item_modifier_lists_max_modifier_lists_to_disable: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of modifier list IDs to be disabled that may be included in
    a single `/v2/catalog/update-item-modifier-lists` request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .order import Order
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CreateOrderRequest(UncheckedBaseModel):
    order: typing.Optional[Order] = pydantic.Field(default=None)
    """
    The order to create. If this field is set, the only other top-level field that can be
    set is the `idempotency_key`.
    """

    idempotency_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    A value you specify that uniquely identifies this
    order among orders you have created.
    
    If you are unsure whether a particular order was created successfully,
    you can try it again with the same idempotency key without
    worrying about creating duplicate orders.
    
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawSubscriptionsClient
from ...types.sort_order import SortOrder
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.webhook_subscription import WebhookSubscription
from ...types.list_webhook_subscriptions_response import ListWebhookSubscriptionsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.webhook_subscription import WebhookSubscriptionParams
from ...types.create_webhook_subscription_response import CreateWebhookSubscriptionResponse
from ...types.get_webhook_subscription_response import GetWebhookSubscriptionResponse
from ...types.update_webhook_subscription_response import UpdateWebhookSubscriptionResponse
from ...types.delete_webhook_subscription_response import DeleteWebhookSubscriptionResponse
from ...types.update_webhook_subscription_signature_key_response import UpdateWebhookSubscriptionSignatureKeyResponse
from ...types.test_webhook_subscription_response import TestWebhookSubscriptionResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawSubscriptionsClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSubscriptionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSubscriptionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        include_disabled: typing.Optional[bool] = None,
        sort_order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[WebhookSubscription]:
        """
        Lists all webhook subscriptions owned by your application.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        include_disabled : typing.Optional[bool]
            Includes disabled [Subscription](entity:WebhookSubscription)s.
            By default, all enabled [Subscription](entity:WebhookSubscription)s are returned.

        sort_order : typing.Optional[SortOrder]
            Sorts the returned list by when the [Subscription](entity:WebhookSubscription) was created with the specified order.
            This field defaults to ASC.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.
            The default value of 100 is also the maximum allowed value.

            Default: 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[WebhookSubscription]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.webhooks.subscriptions.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/webhooks/subscriptions",
            method="GET",
            params={
                "cursor": cursor,
                "include_disabled": include_disabled,
                "sort_order": sort_order,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListWebhookSubscriptionsResponse,
                    construct_type(
                        type_=ListWebhookSubscriptionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    include_disabled=include_disabled,
                    sort_order=sort_order,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.subscriptions
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        subscription: WebhookSubscriptionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateWebhookSubscriptionResponse:
        """
        Creates a webhook subscription.

        Parameters
        ----------
        subscription : WebhookSubscriptionParams
            The [Subscription](entity:WebhookSubscription) to create.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [CreateWebhookSubscription](api-endpoint:WebhookSubscriptions-CreateWebhookSubscription) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateWebhookSubscriptionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.webhooks.subscriptions.create(
            idempotency_key="63f84c6c-2200-4c99-846c-2670a1311fbf",
            subscription={
                "name": "Example Webhook Subscription",
                "event_types": ["payment.created", "payment.updated"],
                "notification_url": "https://example-webhook-url.com",
                "api_version": "2021-12-15",
            },
        )
        """
        response = self._raw_client.create(
            subscription=subscription, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def get(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetWebhookSubscriptionResponse:
        """
        Retrieves a webhook subscription identified by its ID.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWebhookSubscriptionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.webhooks.subscriptions.get(
            subscription_id="subscription_id",
        )
        """
        response = self._raw_client.get(subscription_id, request_options=request_options)
        return response.data

    def update(
        self,
        subscription_id: str,
        *,
        subscription: typing.Optional[WebhookSubscriptionParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWebhookSubscriptionResponse:
        """
        Updates a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        subscription : typing.Optional[WebhookSubscriptionParams]
            The [Subscription](entity:WebhookSubscription) to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWebhookSubscriptionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.webhooks.subscriptions.update(
            subscription_id="subscription_id",
            subscription={
                "name": "Updated Example Webhook Subscription",
                "enabled": False,
            },
        )
        """
        response = self._raw_client.update(subscription_id, subscription=subscription, request_options=request_options)
        return response.data

    def delete(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteWebhookSubscriptionResponse:
        """
        Deletes a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteWebhookSubscriptionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.webhooks.subscriptions.delete(
            subscription_id="subscription_id",
        )
        """
        response = self._raw_client.delete(subscription_id, request_options=request_options)
        return response.data

    def update_signature_key(
        self,
        subscription_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWebhookSubscriptionSignatureKeyResponse:
        """
        Updates a webhook subscription by replacing the existing signature key with a new one.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [UpdateWebhookSubscriptionSignatureKey](api-endpoint:WebhookSubscriptions-UpdateWebhookSubscriptionSignatureKey) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWebhookSubscriptionSignatureKeyResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.webhooks.subscriptions.update_signature_key(
            subscription_id="subscription_id",
            idempotency_key="ed80ae6b-0654-473b-bbab-a39aee89a60d",
        )
        """
        response = self._raw_client.update_signature_key(
            subscription_id, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def test(
        self,
        subscription_id: str,
        *,
        event_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TestWebhookSubscriptionResponse:
        """
        Tests a webhook subscription by sending a test event to the notification URL.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to test.

        event_type : typing.Optional[str]
            The event type that will be used to test the [Subscription](entity:WebhookSubscription). The event type must be
            contained in the list of event types in the [Subscription](entity:WebhookSubscription).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestWebhookSubscriptionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.webhooks.subscriptions.test(
            subscription_id="subscription_id",
            event_type="payment.created",
        )
        """
        response = self._raw_client.test(subscription_id, event_type=event_type, request_options=request_options)
        return response.data


class AsyncSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSubscriptionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSubscriptionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        include_disabled: typing.Optional[bool] = None,
        sort_order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[WebhookSubscription]:
        """
        Lists all webhook subscriptions owned by your application.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        include_disabled : typing.Optional[bool]
            Includes disabled [Subscription](entity:WebhookSubscription)s.
            By default, all enabled [Subscription](entity:WebhookSubscription)s are returned.

        sort_order : typing.Optional[SortOrder]
            Sorts the returned list by when the [Subscription](entity:WebhookSubscription) was created with the specified order.
            This field defaults to ASC.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.
            The default value of 100 is also the maximum allowed value.

            Default: 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[WebhookSubscription]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.webhooks.subscriptions.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/webhooks/subscriptions",
            method="GET",
            params={
                "cursor": cursor,
                "include_disabled": include_disabled,
                "sort_order": sort_order,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListWebhookSubscriptionsResponse,
                    construct_type(
                        type_=ListWebhookSubscriptionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    include_disabled=include_disabled,
                    sort_order=sort_order,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.subscriptions
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        subscription: WebhookSubscriptionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateWebhookSubscriptionResponse:
        """
        Creates a webhook subscription.

        Parameters
        ----------
        subscription : WebhookSubscriptionParams
            The [Subscription](entity:WebhookSubscription) to create.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [CreateWebhookSubscription](api-endpoint:WebhookSubscriptions-CreateWebhookSubscription) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateWebhookSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhooks.subscriptions.create(
                idempotency_key="63f84c6c-2200-4c99-846c-2670a1311fbf",
                subscription={
                    "name": "Example Webhook Subscription",
                    "event_types": ["payment.created", "payment.updated"],
                    "notification_url": "https://example-webhook-url.com",
                    "api_version": "2021-12-15",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            subscription=subscription, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def get(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetWebhookSubscriptionResponse:
        """
        Retrieves a webhook subscription identified by its ID.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWebhookSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhooks.subscriptions.get(
                subscription_id="subscription_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(subscription_id, request_options=request_options)
        return response.data

    async def update(
        self,
        subscription_id: str,
        *,
        subscription: typing.Optional[WebhookSubscriptionParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWebhookSubscriptionResponse:
        """
        Updates a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        subscription : typing.Optional[WebhookSubscriptionParams]
            The [Subscription](entity:WebhookSubscription) to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWebhookSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhooks.subscriptions.update(
                subscription_id="subscription_id",
                subscription={
                    "name": "Updated Example Webhook Subscription",
                    "enabled": False,
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            subscription_id, subscription=subscription, request_options=request_options
        )
        return response.data

    async def delete(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteWebhookSubscriptionResponse:
        """
        Deletes a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteWebhookSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhooks.subscriptions.delete(
                subscription_id="subscription_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(subscription_id, request_options=request_options)
        return response.data

    async def update_signature_key(
        self,
        subscription_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWebhookSubscriptionSignatureKeyResponse:
        """
        Updates a webhook subscription by replacing the existing signature key with a new one.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [UpdateWebhookSubscriptionSignatureKey](api-endpoint:WebhookSubscriptions-UpdateWebhookSubscriptionSignatureKey) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWebhookSubscriptionSignatureKeyResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhooks.subscriptions.update_signature_key(
                subscription_id="subscription_id",
                idempotency_key="ed80ae6b-0654-473b-bbab-a39aee89a60d",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update_signature_key(
            subscription_id, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def test(
        self,
        subscription_id: str,
        *,
        event_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TestWebhookSubscriptionResponse:
        """
        Tests a webhook subscription by sending a test event to the notification URL.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to test.

        event_type : typing.Optional[str]
            The event type that will be used to test the [Subscription](entity:WebhookSubscription). The event type must be
            contained in the list of event types in the [Subscription](entity:WebhookSubscription).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestWebhookSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhooks.subscriptions.test(
                subscription_id="subscription_id",
                event_type="payment.created",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.test(subscription_id, event_type=event_type, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class SaveCardOptionsParams(typing_extensions.TypedDict):
    """
    Describes save-card action fields.
    """

    customer_id: str
    """
    The square-assigned ID of the customer linked to the saved card.
    """

    card_id: typing_extensions.NotRequired[str]
    """
    The id of the created card-on-file.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional user-defined reference ID that can be used to associate
    this `Card` to another entity in an external system. For example, a customer
    ID generated by a third-party system.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomersClient
from .custom_attribute_definitions.client import CustomAttributeDefinitionsClient
from .groups.client import GroupsClient
from .segments.client import SegmentsClient
from .cards.client import CardsClient
from .custom_attributes.client import CustomAttributesClient
from ..types.customer_sort_field import CustomerSortField
from ..types.sort_order import SortOrder
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.customer import Customer
from ..types.list_customers_response import ListCustomersResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.address import AddressParams
from ..requests.customer_tax_ids import CustomerTaxIdsParams
from ..types.create_customer_response import CreateCustomerResponse
from ..requests.bulk_create_customer_data import BulkCreateCustomerDataParams
from ..types.bulk_create_customers_response import BulkCreateCustomersResponse
from ..types.bulk_delete_customers_response import BulkDeleteCustomersResponse
from ..types.bulk_retrieve_customers_response import BulkRetrieveCustomersResponse
from ..requests.bulk_update_customer_data import BulkUpdateCustomerDataParams
from ..types.bulk_update_customers_response import BulkUpdateCustomersResponse
from ..requests.customer_query import CustomerQueryParams
from ..types.search_customers_response import SearchCustomersResponse
from ..types.get_customer_response import GetCustomerResponse
from ..types.update_customer_response import UpdateCustomerResponse
from ..types.delete_customer_response import DeleteCustomerResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomersClient
from .custom_attribute_definitions.client import AsyncCustomAttributeDefinitionsClient
from .groups.client import AsyncGroupsClient
from .segments.client import AsyncSegmentsClient
from .cards.client import AsyncCardsClient
from .custom_attributes.client import AsyncCustomAttributesClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomersClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = CustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.groups = GroupsClient(client_wrapper=client_wrapper)

        self.segments = SegmentsClient(client_wrapper=client_wrapper)

        self.cards = CardsClient(client_wrapper=client_wrapper)

        self.custom_attributes = CustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        sort_field: typing.Optional[CustomerSortField] = None,
        sort_order: typing.Optional[SortOrder] = None,
        count: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Customer]:
        """
        Lists customer profiles associated with a Square account.

        Under normal operating conditions, newly created or updated customer profiles become available
        for the listing operation in well under 30 seconds. Occasionally, propagation of the new or updated
        profiles can take closer to one minute or longer, especially during network incidents and outages.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the specified limit is less than 1 or greater than 100, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 100.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        sort_field : typing.Optional[CustomerSortField]
            Indicates how customers should be sorted.

            The default value is `DEFAULT`.

        sort_order : typing.Optional[SortOrder]
            Indicates whether customers should be sorted in ascending (`ASC`) or
            descending (`DESC`) order.

            The default value is `ASC`.

        count : typing.Optional[bool]
            Indicates whether to return the total count of customers in the `count` field of the response.

            The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Customer]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.customers.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "sort_field": sort_field,
                "sort_order": sort_order,
                "count": count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomersResponse,
                    construct_type(
                        type_=ListCustomersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    sort_field=sort_field,
                    sort_order=sort_order,
                    count=count,
                    request_options=request_options,
                )
                _items = _parsed_response.customers
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        company_name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        email_address: typing.Optional[str] = OMIT,
        address: typing.Optional[AddressParams] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        note: typing.Optional[str] = OMIT,
        birthday: typing.Optional[str] = OMIT,
        tax_ids: typing.Optional[CustomerTaxIdsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerResponse:
        """
        Creates a new customer for a business.

        You must provide at least one of the following values in your request to this
        endpoint:

        - `given_name`
        - `family_name`
        - `company_name`
        - `email_address`
        - `phone_number`

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            The idempotency key for the request.	For more information, see
            [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        given_name : typing.Optional[str]
            The given name (that is, the first name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        family_name : typing.Optional[str]
            The family name (that is, the last name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        company_name : typing.Optional[str]
            A business name associated with the customer profile.

            The maximum length for this value is 500 characters.

        nickname : typing.Optional[str]
            A nickname for the customer profile.

            The maximum length for this value is 100 characters.

        email_address : typing.Optional[str]
            The email address associated with the customer profile.

            The maximum length for this value is 254 characters.

        address : typing.Optional[AddressParams]
            The physical address associated with the customer profile. For maximum length constraints, see
            [Customer addresses](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#address).
            The `first_name` and `last_name` fields are ignored if they are present in the request.

        phone_number : typing.Optional[str]
            The phone number associated with the customer profile. The phone number must be valid and can contain
            9–16 digits, with an optional `+` prefix and country code. For more information, see
            [Customer phone numbers](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#phone-number).

        reference_id : typing.Optional[str]
            An optional second ID used to associate the customer profile with an
            entity in another system.

            The maximum length for this value is 100 characters.

        note : typing.Optional[str]
            A custom note associated with the customer profile.

        birthday : typing.Optional[str]
            The birthday associated with the customer profile, in `YYYY-MM-DD` or `MM-DD` format. For example,
            specify `1998-09-21` for September 21, 1998, or `09-21` for September 21. Birthdays are returned in `YYYY-MM-DD`
            format, where `YYYY` is the specified birth year or `0000` if a birth year is not specified.

        tax_ids : typing.Optional[CustomerTaxIdsParams]
            The tax ID associated with the customer profile. This field is available only for customers of sellers
            in EU countries or the United Kingdom. For more information,
            see [Customer tax IDs](https://developer.squareup.com/docs/customers-api/what-it-does#customer-tax-ids).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.create(
            given_name="Amelia",
            family_name="Earhart",
            email_address="Amelia.Earhart@example.com",
            address={
                "address_line1": "500 Electric Ave",
                "address_line2": "Suite 600",
                "locality": "New York",
                "administrative_district_level1": "NY",
                "postal_code": "10003",
                "country": "US",
            },
            phone_number="+1-212-555-4240",
            reference_id="YOUR_REFERENCE_ID",
            note="a customer",
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key,
            given_name=given_name,
            family_name=family_name,
            company_name=company_name,
            nickname=nickname,
            email_address=email_address,
            address=address,
            phone_number=phone_number,
            reference_id=reference_id,
            note=note,
            birthday=birthday,
            tax_ids=tax_ids,
            request_options=request_options,
        )
        return response.data

    def batch_create(
        self,
        *,
        customers: typing.Dict[str, BulkCreateCustomerDataParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkCreateCustomersResponse:
        """
        Creates multiple [customer profiles](entity:Customer) for a business.

        This endpoint takes a map of individual create requests and returns a map of responses.

        You must provide at least one of the following values in each create request:

        - `given_name`
        - `family_name`
        - `company_name`
        - `email_address`
        - `phone_number`

        Parameters
        ----------
        customers : typing.Dict[str, BulkCreateCustomerDataParams]
            A map of 1 to 100 individual create requests, represented by `idempotency key: { customer data }`
            key-value pairs.

            Each key is an [idempotency key](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency)
            that uniquely identifies the create request. Each value contains the customer data used to create the
            customer profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkCreateCustomersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.batch_create(
            customers={
                "8bb76c4f-e35d-4c5b-90de-1194cd9179f0": {
                    "given_name": "Amelia",
                    "family_name": "Earhart",
                    "email_address": "Amelia.Earhart@example.com",
                    "address": {
                        "address_line1": "500 Electric Ave",
                        "address_line2": "Suite 600",
                        "locality": "New York",
                        "administrative_district_level1": "NY",
                        "postal_code": "10003",
                        "country": "US",
                    },
                    "phone_number": "+1-212-555-4240",
                    "reference_id": "YOUR_REFERENCE_ID",
                    "note": "a customer",
                },
                "d1689f23-b25d-4932-b2f0-aed00f5e2029": {
                    "given_name": "Marie",
                    "family_name": "Curie",
                    "email_address": "Marie.Curie@example.com",
                    "address": {
                        "address_line1": "500 Electric Ave",
                        "address_line2": "Suite 601",
                        "locality": "New York",
                        "administrative_district_level1": "NY",
                        "postal_code": "10003",
                        "country": "US",
                    },
                    "phone_number": "+1-212-444-4240",
                    "reference_id": "YOUR_REFERENCE_ID",
                    "note": "another customer",
                },
            },
        )
        """
        response = self._raw_client.batch_create(customers=customers, request_options=request_options)
        return response.data

    def bulk_delete_customers(
        self, *, customer_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkDeleteCustomersResponse:
        """
        Deletes multiple customer profiles.

        The endpoint takes a list of customer IDs and returns a map of responses.

        Parameters
        ----------
        customer_ids : typing.Sequence[str]
            The IDs of the [customer profiles](entity:Customer) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteCustomersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.bulk_delete_customers(
            customer_ids=[
                "8DDA5NZVBZFGAX0V3HPF81HHE0",
                "N18CPRVXR5214XPBBA6BZQWF3C",
                "2GYD7WNXF7BJZW1PMGNXZ3Y8M8",
            ],
        )
        """
        response = self._raw_client.bulk_delete_customers(customer_ids=customer_ids, request_options=request_options)
        return response.data

    def bulk_retrieve_customers(
        self, *, customer_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkRetrieveCustomersResponse:
        """
        Retrieves multiple customer profiles.

        This endpoint takes a list of customer IDs and returns a map of responses.

        Parameters
        ----------
        customer_ids : typing.Sequence[str]
            The IDs of the [customer profiles](entity:Customer) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkRetrieveCustomersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.bulk_retrieve_customers(
            customer_ids=[
                "8DDA5NZVBZFGAX0V3HPF81HHE0",
                "N18CPRVXR5214XPBBA6BZQWF3C",
                "2GYD7WNXF7BJZW1PMGNXZ3Y8M8",
            ],
        )
        """
        response = self._raw_client.bulk_retrieve_customers(customer_ids=customer_ids, request_options=request_options)
        return response.data

    def bulk_update_customers(
        self,
        *,
        customers: typing.Dict[str, BulkUpdateCustomerDataParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpdateCustomersResponse:
        """
        Updates multiple customer profiles.

        This endpoint takes a map of individual update requests and returns a map of responses.

        Parameters
        ----------
        customers : typing.Dict[str, BulkUpdateCustomerDataParams]
            A map of 1 to 100 individual update requests, represented by `customer ID: { customer data }`
            key-value pairs.

            Each key is the ID of the [customer profile](entity:Customer) to update. To update a customer profile
            that was created by merging existing profiles, provide the ID of the newly created profile.

            Each value contains the updated customer data. Only new or changed fields are required. To add or
            update a field, specify the new value. To remove a field, specify `null`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpdateCustomersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.bulk_update_customers(
            customers={
                "8DDA5NZVBZFGAX0V3HPF81HHE0": {
                    "email_address": "New.Amelia.Earhart@example.com",
                    "note": "updated customer note",
                    "version": 2,
                },
                "N18CPRVXR5214XPBBA6BZQWF3C": {
                    "given_name": "Marie",
                    "family_name": "Curie",
                    "version": 0,
                },
            },
        )
        """
        response = self._raw_client.bulk_update_customers(customers=customers, request_options=request_options)
        return response.data

    def search(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[CustomerQueryParams] = OMIT,
        count: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchCustomersResponse:
        """
        Searches the customer profiles associated with a Square account using one or more supported query filters.

        Calling `SearchCustomers` without any explicit query filter returns all
        customer profiles ordered alphabetically based on `given_name` and
        `family_name`.

        Under normal operating conditions, newly created or updated customer profiles become available
        for the search operation in well under 30 seconds. Occasionally, propagation of the new or updated
        profiles can take closer to one minute or longer, especially during network incidents and outages.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Include the pagination cursor in subsequent calls to this endpoint to retrieve
            the next set of results associated with the original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the specified limit is invalid, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 100.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        query : typing.Optional[CustomerQueryParams]
            The filtering and sorting criteria for the search request. If a query is not specified,
            Square returns all customer profiles ordered alphabetically by `given_name` and `family_name`.

        count : typing.Optional[bool]
            Indicates whether to return the total count of matching customers in the `count` field of the response.

            The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchCustomersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.search(
            limit=2,
            query={
                "filter": {
                    "creation_source": {"values": ["THIRD_PARTY"], "rule": "INCLUDE"},
                    "created_at": {
                        "start_at": "2018-01-01T00:00:00-00:00",
                        "end_at": "2018-02-01T00:00:00-00:00",
                    },
                    "email_address": {"fuzzy": "example.com"},
                    "group_ids": {"all_": ["545AXB44B4XXWMVQ4W8SBT3HHF"]},
                },
                "sort": {"field": "CREATED_AT", "order": "ASC"},
            },
        )
        """
        response = self._raw_client.search(
            cursor=cursor, limit=limit, query=query, count=count, request_options=request_options
        )
        return response.data

    def get(self, customer_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetCustomerResponse:
        """
        Returns details for a single customer.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.get(
            customer_id="customer_id",
        )
        """
        response = self._raw_client.get(customer_id, request_options=request_options)
        return response.data

    def update(
        self,
        customer_id: str,
        *,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        company_name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        email_address: typing.Optional[str] = OMIT,
        address: typing.Optional[AddressParams] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        note: typing.Optional[str] = OMIT,
        birthday: typing.Optional[str] = OMIT,
        version: typing.Optional[int] = OMIT,
        tax_ids: typing.Optional[CustomerTaxIdsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCustomerResponse:
        """
        Updates a customer profile. This endpoint supports sparse updates, so only new or changed fields are required in the request.
        To add or update a field, specify the new value. To remove a field, specify `null`.

        To update a customer profile that was created by merging existing profiles, you must use the ID of the newly created profile.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to update.

        given_name : typing.Optional[str]
            The given name (that is, the first name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        family_name : typing.Optional[str]
            The family name (that is, the last name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        company_name : typing.Optional[str]
            A business name associated with the customer profile.

            The maximum length for this value is 500 characters.

        nickname : typing.Optional[str]
            A nickname for the customer profile.

            The maximum length for this value is 100 characters.

        email_address : typing.Optional[str]
            The email address associated with the customer profile.

            The maximum length for this value is 254 characters.

        address : typing.Optional[AddressParams]
            The physical address associated with the customer profile. Only new or changed fields are required in the request.

            For maximum length constraints, see [Customer addresses](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#address).
            The `first_name` and `last_name` fields are ignored if they are present in the request.

        phone_number : typing.Optional[str]
            The phone number associated with the customer profile. The phone number must be valid and can contain
            9–16 digits, with an optional `+` prefix and country code. For more information, see
            [Customer phone numbers](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#phone-number).

        reference_id : typing.Optional[str]
            An optional second ID used to associate the customer profile with an
            entity in another system.

            The maximum length for this value is 100 characters.

        note : typing.Optional[str]
            A custom note associated with the customer profile.

        birthday : typing.Optional[str]
            The birthday associated with the customer profile, in `YYYY-MM-DD` or `MM-DD` format. For example,
            specify `1998-09-21` for September 21, 1998, or `09-21` for September 21. Birthdays are returned in `YYYY-MM-DD`
            format, where `YYYY` is the specified birth year or `0000` if a birth year is not specified.

        version : typing.Optional[int]
            The current version of the customer profile.

            As a best practice, you should include this field to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control. For more information, see [Update a customer profile](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#update-a-customer-profile).

        tax_ids : typing.Optional[CustomerTaxIdsParams]
            The tax ID associated with the customer profile. This field is available only for customers of sellers
            in EU countries or the United Kingdom. For more information,
            see [Customer tax IDs](https://developer.squareup.com/docs/customers-api/what-it-does#customer-tax-ids).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomerResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.update(
            customer_id="customer_id",
            email_address="New.Amelia.Earhart@example.com",
            note="updated customer note",
            version=2,
        )
        """
        response = self._raw_client.update(
            customer_id,
            given_name=given_name,
            family_name=family_name,
            company_name=company_name,
            nickname=nickname,
            email_address=email_address,
            address=address,
            phone_number=phone_number,
            reference_id=reference_id,
            note=note,
            birthday=birthday,
            version=version,
            tax_ids=tax_ids,
            request_options=request_options,
        )
        return response.data

    def delete(
        self,
        customer_id: str,
        *,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteCustomerResponse:
        """
        Deletes a customer profile from a business.

        To delete a customer profile that was created by merging existing profiles, you must use the ID of the newly created profile.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to delete.

        version : typing.Optional[int]
            The current version of the customer profile.

            As a best practice, you should include this parameter to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control.  For more information, see [Delete a customer profile](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#delete-customer-profile).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.delete(
            customer_id="customer_id",
        )
        """
        response = self._raw_client.delete(customer_id, version=version, request_options=request_options)
        return response.data


class AsyncCustomersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomersClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = AsyncCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.groups = AsyncGroupsClient(client_wrapper=client_wrapper)

        self.segments = AsyncSegmentsClient(client_wrapper=client_wrapper)

        self.cards = AsyncCardsClient(client_wrapper=client_wrapper)

        self.custom_attributes = AsyncCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        sort_field: typing.Optional[CustomerSortField] = None,
        sort_order: typing.Optional[SortOrder] = None,
        count: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Customer]:
        """
        Lists customer profiles associated with a Square account.

        Under normal operating conditions, newly created or updated customer profiles become available
        for the listing operation in well under 30 seconds. Occasionally, propagation of the new or updated
        profiles can take closer to one minute or longer, especially during network incidents and outages.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the specified limit is less than 1 or greater than 100, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 100.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        sort_field : typing.Optional[CustomerSortField]
            Indicates how customers should be sorted.

            The default value is `DEFAULT`.

        sort_order : typing.Optional[SortOrder]
            Indicates whether customers should be sorted in ascending (`ASC`) or
            descending (`DESC`) order.

            The default value is `ASC`.

        count : typing.Optional[bool]
            Indicates whether to return the total count of customers in the `count` field of the response.

            The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Customer]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.customers.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "sort_field": sort_field,
                "sort_order": sort_order,
                "count": count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomersResponse,
                    construct_type(
                        type_=ListCustomersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    sort_field=sort_field,
                    sort_order=sort_order,
                    count=count,
                    request_options=request_options,
                )
                _items = _parsed_response.customers
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        company_name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        email_address: typing.Optional[str] = OMIT,
        address: typing.Optional[AddressParams] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        note: typing.Optional[str] = OMIT,
        birthday: typing.Optional[str] = OMIT,
        tax_ids: typing.Optional[CustomerTaxIdsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerResponse:
        """
        Creates a new customer for a business.

        You must provide at least one of the following values in your request to this
        endpoint:

        - `given_name`
        - `family_name`
        - `company_name`
        - `email_address`
        - `phone_number`

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            The idempotency key for the request.	For more information, see
            [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        given_name : typing.Optional[str]
            The given name (that is, the first name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        family_name : typing.Optional[str]
            The family name (that is, the last name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        company_name : typing.Optional[str]
            A business name associated with the customer profile.

            The maximum length for this value is 500 characters.

        nickname : typing.Optional[str]
            A nickname for the customer profile.

            The maximum length for this value is 100 characters.

        email_address : typing.Optional[str]
            The email address associated with the customer profile.

            The maximum length for this value is 254 characters.

        address : typing.Optional[AddressParams]
            The physical address associated with the customer profile. For maximum length constraints, see
            [Customer addresses](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#address).
            The `first_name` and `last_name` fields are ignored if they are present in the request.

        phone_number : typing.Optional[str]
            The phone number associated with the customer profile. The phone number must be valid and can contain
            9–16 digits, with an optional `+` prefix and country code. For more information, see
            [Customer phone numbers](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#phone-number).

        reference_id : typing.Optional[str]
            An optional second ID used to associate the customer profile with an
            entity in another system.

            The maximum length for this value is 100 characters.

        note : typing.Optional[str]
            A custom note associated with the customer profile.

        birthday : typing.Optional[str]
            The birthday associated with the customer profile, in `YYYY-MM-DD` or `MM-DD` format. For example,
            specify `1998-09-21` for September 21, 1998, or `09-21` for September 21. Birthdays are returned in `YYYY-MM-DD`
            format, where `YYYY` is the specified birth year or `0000` if a birth year is not specified.

        tax_ids : typing.Optional[CustomerTaxIdsParams]
            The tax ID associated with the customer profile. This field is available only for customers of sellers
            in EU countries or the United Kingdom. For more information,
            see [Customer tax IDs](https://developer.squareup.com/docs/customers-api/what-it-does#customer-tax-ids).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.create(
                given_name="Amelia",
                family_name="Earhart",
                email_address="Amelia.Earhart@example.com",
                address={
                    "address_line1": "500 Electric Ave",
                    "address_line2": "Suite 600",
                    "locality": "New York",
                    "administrative_district_level1": "NY",
                    "postal_code": "10003",
                    "country": "US",
                },
                phone_number="+1-212-555-4240",
                reference_id="YOUR_REFERENCE_ID",
                note="a customer",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key,
            given_name=given_name,
            family_name=family_name,
            company_name=company_name,
            nickname=nickname,
            email_address=email_address,
            address=address,
            phone_number=phone_number,
            reference_id=reference_id,
            note=note,
            birthday=birthday,
            tax_ids=tax_ids,
            request_options=request_options,
        )
        return response.data

    async def batch_create(
        self,
        *,
        customers: typing.Dict[str, BulkCreateCustomerDataParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkCreateCustomersResponse:
        """
        Creates multiple [customer profiles](entity:Customer) for a business.

        This endpoint takes a map of individual create requests and returns a map of responses.

        You must provide at least one of the following values in each create request:

        - `given_name`
        - `family_name`
        - `company_name`
        - `email_address`
        - `phone_number`

        Parameters
        ----------
        customers : typing.Dict[str, BulkCreateCustomerDataParams]
            A map of 1 to 100 individual create requests, represented by `idempotency key: { customer data }`
            key-value pairs.

            Each key is an [idempotency key](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency)
            that uniquely identifies the create request. Each value contains the customer data used to create the
            customer profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkCreateCustomersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.batch_create(
                customers={
                    "8bb76c4f-e35d-4c5b-90de-1194cd9179f0": {
                        "given_name": "Amelia",
                        "family_name": "Earhart",
                        "email_address": "Amelia.Earhart@example.com",
                        "address": {
                            "address_line1": "500 Electric Ave",
                            "address_line2": "Suite 600",
                            "locality": "New York",
                            "administrative_district_level1": "NY",
                            "postal_code": "10003",
                            "country": "US",
                        },
                        "phone_number": "+1-212-555-4240",
                        "reference_id": "YOUR_REFERENCE_ID",
                        "note": "a customer",
                    },
                    "d1689f23-b25d-4932-b2f0-aed00f5e2029": {
                        "given_name": "Marie",
                        "family_name": "Curie",
                        "email_address": "Marie.Curie@example.com",
                        "address": {
                            "address_line1": "500 Electric Ave",
                            "address_line2": "Suite 601",
                            "locality": "New York",
                            "administrative_district_level1": "NY",
                            "postal_code": "10003",
                            "country": "US",
                        },
                        "phone_number": "+1-212-444-4240",
                        "reference_id": "YOUR_REFERENCE_ID",
                        "note": "another customer",
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_create(customers=customers, request_options=request_options)
        return response.data

    async def bulk_delete_customers(
        self, *, customer_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkDeleteCustomersResponse:
        """
        Deletes multiple customer profiles.

        The endpoint takes a list of customer IDs and returns a map of responses.

        Parameters
        ----------
        customer_ids : typing.Sequence[str]
            The IDs of the [customer profiles](entity:Customer) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteCustomersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.bulk_delete_customers(
                customer_ids=[
                    "8DDA5NZVBZFGAX0V3HPF81HHE0",
                    "N18CPRVXR5214XPBBA6BZQWF3C",
                    "2GYD7WNXF7BJZW1PMGNXZ3Y8M8",
                ],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.bulk_delete_customers(
            customer_ids=customer_ids, request_options=request_options
        )
        return response.data

    async def bulk_retrieve_customers(
        self, *, customer_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> BulkRetrieveCustomersResponse:
        """
        Retrieves multiple customer profiles.

        This endpoint takes a list of customer IDs and returns a map of responses.

        Parameters
        ----------
        customer_ids : typing.Sequence[str]
            The IDs of the [customer profiles](entity:Customer) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkRetrieveCustomersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.bulk_retrieve_customers(
                customer_ids=[
                    "8DDA5NZVBZFGAX0V3HPF81HHE0",
                    "N18CPRVXR5214XPBBA6BZQWF3C",
                    "2GYD7WNXF7BJZW1PMGNXZ3Y8M8",
                ],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.bulk_retrieve_customers(
            customer_ids=customer_ids, request_options=request_options
        )
        return response.data

    async def bulk_update_customers(
        self,
        *,
        customers: typing.Dict[str, BulkUpdateCustomerDataParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpdateCustomersResponse:
        """
        Updates multiple customer profiles.

        This endpoint takes a map of individual update requests and returns a map of responses.

        Parameters
        ----------
        customers : typing.Dict[str, BulkUpdateCustomerDataParams]
            A map of 1 to 100 individual update requests, represented by `customer ID: { customer data }`
            key-value pairs.

            Each key is the ID of the [customer profile](entity:Customer) to update. To update a customer profile
            that was created by merging existing profiles, provide the ID of the newly created profile.

            Each value contains the updated customer data. Only new or changed fields are required. To add or
            update a field, specify the new value. To remove a field, specify `null`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpdateCustomersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.bulk_update_customers(
                customers={
                    "8DDA5NZVBZFGAX0V3HPF81HHE0": {
                        "email_address": "New.Amelia.Earhart@example.com",
                        "note": "updated customer note",
                        "version": 2,
                    },
                    "N18CPRVXR5214XPBBA6BZQWF3C": {
                        "given_name": "Marie",
                        "family_name": "Curie",
                        "version": 0,
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.bulk_update_customers(customers=customers, request_options=request_options)
        return response.data

    async def search(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[CustomerQueryParams] = OMIT,
        count: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchCustomersResponse:
        """
        Searches the customer profiles associated with a Square account using one or more supported query filters.

        Calling `SearchCustomers` without any explicit query filter returns all
        customer profiles ordered alphabetically based on `given_name` and
        `family_name`.

        Under normal operating conditions, newly created or updated customer profiles become available
        for the search operation in well under 30 seconds. Occasionally, propagation of the new or updated
        profiles can take closer to one minute or longer, especially during network incidents and outages.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Include the pagination cursor in subsequent calls to this endpoint to retrieve
            the next set of results associated with the original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the specified limit is invalid, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 100.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        query : typing.Optional[CustomerQueryParams]
            The filtering and sorting criteria for the search request. If a query is not specified,
            Square returns all customer profiles ordered alphabetically by `given_name` and `family_name`.

        count : typing.Optional[bool]
            Indicates whether to return the total count of matching customers in the `count` field of the response.

            The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchCustomersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.search(
                limit=2,
                query={
                    "filter": {
                        "creation_source": {
                            "values": ["THIRD_PARTY"],
                            "rule": "INCLUDE",
                        },
                        "created_at": {
                            "start_at": "2018-01-01T00:00:00-00:00",
                            "end_at": "2018-02-01T00:00:00-00:00",
                        },
                        "email_address": {"fuzzy": "example.com"},
                        "group_ids": {"all_": ["545AXB44B4XXWMVQ4W8SBT3HHF"]},
                    },
                    "sort": {"field": "CREATED_AT", "order": "ASC"},
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            cursor=cursor, limit=limit, query=query, count=count, request_options=request_options
        )
        return response.data

    async def get(
        self, customer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomerResponse:
        """
        Returns details for a single customer.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.get(
                customer_id="customer_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(customer_id, request_options=request_options)
        return response.data

    async def update(
        self,
        customer_id: str,
        *,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        company_name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        email_address: typing.Optional[str] = OMIT,
        address: typing.Optional[AddressParams] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        note: typing.Optional[str] = OMIT,
        birthday: typing.Optional[str] = OMIT,
        version: typing.Optional[int] = OMIT,
        tax_ids: typing.Optional[CustomerTaxIdsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCustomerResponse:
        """
        Updates a customer profile. This endpoint supports sparse updates, so only new or changed fields are required in the request.
        To add or update a field, specify the new value. To remove a field, specify `null`.

        To update a customer profile that was created by merging existing profiles, you must use the ID of the newly created profile.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to update.

        given_name : typing.Optional[str]
            The given name (that is, the first name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        family_name : typing.Optional[str]
            The family name (that is, the last name) associated with the customer profile.

            The maximum length for this value is 300 characters.

        company_name : typing.Optional[str]
            A business name associated with the customer profile.

            The maximum length for this value is 500 characters.

        nickname : typing.Optional[str]
            A nickname for the customer profile.

            The maximum length for this value is 100 characters.

        email_address : typing.Optional[str]
            The email address associated with the customer profile.

            The maximum length for this value is 254 characters.

        address : typing.Optional[AddressParams]
            The physical address associated with the customer profile. Only new or changed fields are required in the request.

            For maximum length constraints, see [Customer addresses](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#address).
            The `first_name` and `last_name` fields are ignored if they are present in the request.

        phone_number : typing.Optional[str]
            The phone number associated with the customer profile. The phone number must be valid and can contain
            9–16 digits, with an optional `+` prefix and country code. For more information, see
            [Customer phone numbers](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#phone-number).

        reference_id : typing.Optional[str]
            An optional second ID used to associate the customer profile with an
            entity in another system.

            The maximum length for this value is 100 characters.

        note : typing.Optional[str]
            A custom note associated with the customer profile.

        birthday : typing.Optional[str]
            The birthday associated with the customer profile, in `YYYY-MM-DD` or `MM-DD` format. For example,
            specify `1998-09-21` for September 21, 1998, or `09-21` for September 21. Birthdays are returned in `YYYY-MM-DD`
            format, where `YYYY` is the specified birth year or `0000` if a birth year is not specified.

        version : typing.Optional[int]
            The current version of the customer profile.

            As a best practice, you should include this field to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control. For more information, see [Update a customer profile](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#update-a-customer-profile).

        tax_ids : typing.Optional[CustomerTaxIdsParams]
            The tax ID associated with the customer profile. This field is available only for customers of sellers
            in EU countries or the United Kingdom. For more information,
            see [Customer tax IDs](https://developer.squareup.com/docs/customers-api/what-it-does#customer-tax-ids).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomerResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.update(
                customer_id="customer_id",
                email_address="New.Amelia.Earhart@example.com",
                note="updated customer note",
                version=2,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            customer_id,
            given_name=given_name,
            family_name=family_name,
            company_name=company_name,
            nickname=nickname,
            email_address=email_address,
            address=address,
            phone_number=phone_number,
            reference_id=reference_id,
            note=note,
            birthday=birthday,
            version=version,
            tax_ids=tax_ids,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self,
        customer_id: str,
        *,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteCustomerResponse:
        """
        Deletes a customer profile from a business.

        To delete a customer profile that was created by merging existing profiles, you must use the ID of the newly created profile.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to delete.

        version : typing.Optional[int]
            The current version of the customer profile.

            As a best practice, you should include this parameter to enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control.  For more information, see [Delete a customer profile](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#delete-customer-profile).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.delete(
                customer_id="customer_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(customer_id, version=version, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.invoice_payment_reminder_status import InvoicePaymentReminderStatus


class InvoicePaymentReminderParams(typing_extensions.TypedDict):
    """
    Describes a payment request reminder (automatic notification) that Square sends
    to the customer. You configure a reminder relative to the payment request
    `due_date`.
    """

    uid: typing_extensions.NotRequired[str]
    """
    A Square-assigned ID that uniquely identifies the reminder within the
    `InvoicePaymentRequest`.
    """

    relative_scheduled_days: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The number of days before (a negative number) or after (a positive number)
    the payment request `due_date` when the reminder is sent. For example, -3 indicates that
    the reminder should be sent 3 days before the payment request `due_date`.
    """

    message: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The reminder message.
    """

    status: typing_extensions.NotRequired[InvoicePaymentReminderStatus]
    """
    The status of the reminder.
    See [InvoicePaymentReminderStatus](#type-invoicepaymentreminderstatus) for possible values
    """

    sent_at: typing_extensions.NotRequired[str]
    """
    If sent, the timestamp when the reminder was sent, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class FulfillmentPickupDetailsCurbsidePickupDetailsParams(typing_extensions.TypedDict):
    """
    Specific details for curbside pickup.
    """

    curbside_details: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Specific details for curbside pickup, such as parking number and vehicle model.
    """

    buyer_arrived_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    indicating when the buyer arrived and is waiting for pickup. The timestamp must be in RFC 3339 format
    (for example, "2016-09-04T23:59:33.123Z").
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawRewardsClient
from ...requests.loyalty_reward import LoyaltyRewardParams
from ...core.request_options import RequestOptions
from ...types.create_loyalty_reward_response import CreateLoyaltyRewardResponse
from ...requests.search_loyalty_rewards_request_loyalty_reward_query import (
    SearchLoyaltyRewardsRequestLoyaltyRewardQueryParams,
)
from ...types.search_loyalty_rewards_response import SearchLoyaltyRewardsResponse
from ...types.get_loyalty_reward_response import GetLoyaltyRewardResponse
from ...types.delete_loyalty_reward_response import DeleteLoyaltyRewardResponse
from ...types.redeem_loyalty_reward_response import RedeemLoyaltyRewardResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawRewardsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RewardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRewardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRewardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRewardsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        reward: LoyaltyRewardParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLoyaltyRewardResponse:
        """
        Creates a loyalty reward. In the process, the endpoint does following:

        - Uses the `reward_tier_id` in the request to determine the number of points
        to lock for this reward.
        - If the request includes `order_id`, it adds the reward and related discount to the order.

        After a reward is created, the points are locked and
        not available for the buyer to redeem another reward.

        Parameters
        ----------
        reward : LoyaltyRewardParams
            The reward to create.

        idempotency_key : str
            A unique string that identifies this `CreateLoyaltyReward` request.
            Keys can be any valid string, but must be unique for every request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLoyaltyRewardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.rewards.create(
            reward={
                "loyalty_account_id": "5adcb100-07f1-4ee7-b8c6-6bb9ebc474bd",
                "reward_tier_id": "e1b39225-9da5-43d1-a5db-782cdd8ad94f",
                "order_id": "RFZfrdtm3mhO1oGzf5Cx7fEMsmGZY",
            },
            idempotency_key="18c2e5ea-a620-4b1f-ad60-7b167285e451",
        )
        """
        response = self._raw_client.create(
            reward=reward, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        query: typing.Optional[SearchLoyaltyRewardsRequestLoyaltyRewardQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchLoyaltyRewardsResponse:
        """
        Searches for loyalty rewards. This endpoint accepts a request with no query filters and returns results for all loyalty accounts.
        If you include a `query` object, `loyalty_account_id` is required and `status` is  optional.

        If you know a reward ID, use the
        [RetrieveLoyaltyReward](api-endpoint:Loyalty-RetrieveLoyaltyReward) endpoint.

        Search results are sorted by `updated_at` in descending order.

        Parameters
        ----------
        query : typing.Optional[SearchLoyaltyRewardsRequestLoyaltyRewardQueryParams]
            The search criteria for the request.
            If empty, the endpoint retrieves all loyalty rewards in the loyalty program.

        limit : typing.Optional[int]
            The maximum number of results to return in the response. The default value is 30.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to
            this endpoint. Provide this to retrieve the next set of
            results for the original query.
            For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchLoyaltyRewardsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.rewards.search(
            query={"loyalty_account_id": "5adcb100-07f1-4ee7-b8c6-6bb9ebc474bd"},
            limit=10,
        )
        """
        response = self._raw_client.search(query=query, limit=limit, cursor=cursor, request_options=request_options)
        return response.data

    def get(
        self, reward_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyRewardResponse:
        """
        Retrieves a loyalty reward.

        Parameters
        ----------
        reward_id : str
            The ID of the [loyalty reward](entity:LoyaltyReward) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyRewardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.rewards.get(
            reward_id="reward_id",
        )
        """
        response = self._raw_client.get(reward_id, request_options=request_options)
        return response.data

    def delete(
        self, reward_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteLoyaltyRewardResponse:
        """
        Deletes a loyalty reward by doing the following:

        - Returns the loyalty points back to the loyalty account.
        - If an order ID was specified when the reward was created
        (see [CreateLoyaltyReward](api-endpoint:Loyalty-CreateLoyaltyReward)),
        it updates the order by removing the reward and related
        discounts.

        You cannot delete a reward that has reached the terminal state (REDEEMED).

        Parameters
        ----------
        reward_id : str
            The ID of the [loyalty reward](entity:LoyaltyReward) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteLoyaltyRewardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.rewards.delete(
            reward_id="reward_id",
        )
        """
        response = self._raw_client.delete(reward_id, request_options=request_options)
        return response.data

    def redeem(
        self,
        reward_id: str,
        *,
        idempotency_key: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RedeemLoyaltyRewardResponse:
        """
        Redeems a loyalty reward.

        The endpoint sets the reward to the `REDEEMED` terminal state.

        If you are using your own order processing system (not using the
        Orders API), you call this endpoint after the buyer paid for the
        purchase.

        After the reward reaches the terminal state, it cannot be deleted.
        In other words, points used for the reward cannot be returned
        to the account.

        Parameters
        ----------
        reward_id : str
            The ID of the [loyalty reward](entity:LoyaltyReward) to redeem.

        idempotency_key : str
            A unique string that identifies this `RedeemLoyaltyReward` request.
            Keys can be any valid string, but must be unique for every request.

        location_id : str
            The ID of the [location](entity:Location) where the reward is redeemed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedeemLoyaltyRewardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.rewards.redeem(
            reward_id="reward_id",
            idempotency_key="98adc7f7-6963-473b-b29c-f3c9cdd7d994",
            location_id="P034NEENMD09F",
        )
        """
        response = self._raw_client.redeem(
            reward_id, idempotency_key=idempotency_key, location_id=location_id, request_options=request_options
        )
        return response.data


class AsyncRewardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRewardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRewardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRewardsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        reward: LoyaltyRewardParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLoyaltyRewardResponse:
        """
        Creates a loyalty reward. In the process, the endpoint does following:

        - Uses the `reward_tier_id` in the request to determine the number of points
        to lock for this reward.
        - If the request includes `order_id`, it adds the reward and related discount to the order.

        After a reward is created, the points are locked and
        not available for the buyer to redeem another reward.

        Parameters
        ----------
        reward : LoyaltyRewardParams
            The reward to create.

        idempotency_key : str
            A unique string that identifies this `CreateLoyaltyReward` request.
            Keys can be any valid string, but must be unique for every request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLoyaltyRewardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.rewards.create(
                reward={
                    "loyalty_account_id": "5adcb100-07f1-4ee7-b8c6-6bb9ebc474bd",
                    "reward_tier_id": "e1b39225-9da5-43d1-a5db-782cdd8ad94f",
                    "order_id": "RFZfrdtm3mhO1oGzf5Cx7fEMsmGZY",
                },
                idempotency_key="18c2e5ea-a620-4b1f-ad60-7b167285e451",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            reward=reward, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        query: typing.Optional[SearchLoyaltyRewardsRequestLoyaltyRewardQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchLoyaltyRewardsResponse:
        """
        Searches for loyalty rewards. This endpoint accepts a request with no query filters and returns results for all loyalty accounts.
        If you include a `query` object, `loyalty_account_id` is required and `status` is  optional.

        If you know a reward ID, use the
        [RetrieveLoyaltyReward](api-endpoint:Loyalty-RetrieveLoyaltyReward) endpoint.

        Search results are sorted by `updated_at` in descending order.

        Parameters
        ----------
        query : typing.Optional[SearchLoyaltyRewardsRequestLoyaltyRewardQueryParams]
            The search criteria for the request.
            If empty, the endpoint retrieves all loyalty rewards in the loyalty program.

        limit : typing.Optional[int]
            The maximum number of results to return in the response. The default value is 30.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to
            this endpoint. Provide this to retrieve the next set of
            results for the original query.
            For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchLoyaltyRewardsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.rewards.search(
                query={"loyalty_account_id": "5adcb100-07f1-4ee7-b8c6-6bb9ebc474bd"},
                limit=10,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, limit=limit, cursor=cursor, request_options=request_options
        )
        return response.data

    async def get(
        self, reward_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyRewardResponse:
        """
        Retrieves a loyalty reward.

        Parameters
        ----------
        reward_id : str
            The ID of the [loyalty reward](entity:LoyaltyReward) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyRewardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.rewards.get(
                reward_id="reward_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(reward_id, request_options=request_options)
        return response.data

    async def delete(
        self, reward_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteLoyaltyRewardResponse:
        """
        Deletes a loyalty reward by doing the following:

        - Returns the loyalty points back to the loyalty account.
        - If an order ID was specified when the reward was created
        (see [CreateLoyaltyReward](api-endpoint:Loyalty-CreateLoyaltyReward)),
        it updates the order by removing the reward and related
        discounts.

        You cannot delete a reward that has reached the terminal state (REDEEMED).

        Parameters
        ----------
        reward_id : str
            The ID of the [loyalty reward](entity:LoyaltyReward) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteLoyaltyRewardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.rewards.delete(
                reward_id="reward_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(reward_id, request_options=request_options)
        return response.data

    async def redeem(
        self,
        reward_id: str,
        *,
        idempotency_key: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RedeemLoyaltyRewardResponse:
        """
        Redeems a loyalty reward.

        The endpoint sets the reward to the `REDEEMED` terminal state.

        If you are using your own order processing system (not using the
        Orders API), you call this endpoint after the buyer paid for the
        purchase.

        After the reward reaches the terminal state, it cannot be deleted.
        In other words, points used for the reward cannot be returned
        to the account.

        Parameters
        ----------
        reward_id : str
            The ID of the [loyalty reward](entity:LoyaltyReward) to redeem.

        idempotency_key : str
            A unique string that identifies this `RedeemLoyaltyReward` request.
            Keys can be any valid string, but must be unique for every request.

        location_id : str
            The ID of the [location](entity:Location) where the reward is redeemed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedeemLoyaltyRewardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.rewards.redeem(
                reward_id="reward_id",
                idempotency_key="98adc7f7-6963-473b-b29c-f3c9cdd7d994",
                location_id="P034NEENMD09F",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.redeem(
            reward_id, idempotency_key=idempotency_key, location_id=location_id, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .time_range import TimeRangeParams
from ..types.terminal_action_action_type import TerminalActionActionType


class TerminalActionQueryFilterParams(typing_extensions.TypedDict):
    device_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    `TerminalAction`s associated with a specific device. If no device is specified then all
    `TerminalAction`s for the merchant will be displayed.
    """

    created_at: typing_extensions.NotRequired[TimeRangeParams]
    """
    Time range for the beginning of the reporting period. Inclusive.
    Default value: The current time minus one day.
    Note that `TerminalAction`s are available for 30 days after creation.
    """

    status: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Filter results with the desired status of the `TerminalAction`
    Options: `PENDING`, `IN_PROGRESS`, `CANCEL_REQUESTED`, `CANCELED`, `COMPLETED`
    """

    type: typing_extensions.NotRequired[TerminalActionActionType]
    """
    Filter results with the requested ActionType.
    See [TerminalActionActionType](#type-terminalactionactiontype) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CustomerTaxIdsParams(typing_extensions.TypedDict):
    """
    Represents the tax ID associated with a [customer profile](entity:Customer). The corresponding `tax_ids` field is available only for customers of sellers in EU countries or the United Kingdom.
    For more information, see [Customer tax IDs](https://developer.squareup.com/docs/customers-api/what-it-does#customer-tax-ids).
    """

    eu_vat: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The EU VAT identification number for the customer. For example, `IE3426675K`. The ID can contain alphanumeric characters only.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_object import CatalogObjectParams


class SearchCatalogItemsResponseParams(typing_extensions.TypedDict):
    """
    Defines the response body returned from the [SearchCatalogItems](api-endpoint:Catalog-SearchCatalogItems) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    items: typing_extensions.NotRequired[typing.Sequence[CatalogObjectParams]]
    """
    Returned items matching the specified query expressions.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    Pagination token used in the next request to return more of the search result.
    """

    matched_variation_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    Ids of returned item variations matching the specified query expression.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..core.serialization import FieldMetadata


class CatalogV1IdParams(typing_extensions.TypedDict):
    """
    A Square API V1 identifier of an item, including the object ID and its associated location ID.
    """

    catalog_v1id: typing_extensions.NotRequired[
        typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="catalog_v1_id")]
    ]
    """
    The ID for an object used in the Square API V1, if the object ID differs from the Square API V2 object ID.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the `Location` this Connect V1 ID is associated with.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class AchDetailsParams(typing_extensions.TypedDict):
    """
    ACH-specific details about `BANK_ACCOUNT` type payments with the `transfer_type` of `ACH`.
    """

    routing_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The routing number for the bank account.
    """

    account_number_suffix: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The last few digits of the bank account number.
    """

    account_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The type of the bank account performing the transfer. The account type can be `CHECKING`,
    `SAVINGS`, or `UNKNOWN`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyProgramExpirationPolicyParams(typing_extensions.TypedDict):
    """
    Describes when the loyalty program expires.
    """

    expiration_duration: str
    """
    The number of months before points expire, in `P[n]M` RFC 3339 duration format. For example, a value of `P12M` represents a duration of 12 months. 
    Points are valid through the last day of the month in which they are scheduled to expire. For example, with a  `P12M` duration, points earned on July 6, 2020 expire on August 1, 2021.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class OrderRewardParams(typing_extensions.TypedDict):
    """
    Represents a reward that can be applied to an order if the necessary
    reward tier criteria are met. Rewards are created through the Loyalty API.
    """

    id: str
    """
    The identifier of the reward.
    """

    reward_tier_id: str
    """
    The identifier of the reward tier corresponding to this reward.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.tender_card_details_status import TenderCardDetailsStatus
from .card import CardParams
from ..types.tender_card_details_entry_method import TenderCardDetailsEntryMethod


class TenderCardDetailsParams(typing_extensions.TypedDict):
    """
    Represents additional details of a tender with `type` `CARD` or `SQUARE_GIFT_CARD`
    """

    status: typing_extensions.NotRequired[TenderCardDetailsStatus]
    """
    The credit card payment's current state (such as `AUTHORIZED` or
    `CAPTURED`). See [TenderCardDetailsStatus](entity:TenderCardDetailsStatus)
    for possible values.
    See [TenderCardDetailsStatus](#type-tendercarddetailsstatus) for possible values
    """

    card: typing_extensions.NotRequired[CardParams]
    """
    The credit card's non-confidential details.
    """

    entry_method: typing_extensions.NotRequired[TenderCardDetailsEntryMethod]
    """
    The method used to enter the card's details for the transaction.
    See [TenderCardDetailsEntryMethod](#type-tendercarddetailsentrymethod) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.inventory_state import InventoryState
from .money import MoneyParams
from .source_application import SourceApplicationParams
from .inventory_adjustment_group import InventoryAdjustmentGroupParams


class InventoryAdjustmentParams(typing_extensions.TypedDict):
    """
    Represents a change in state or quantity of product inventory at a
    particular time and location.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique ID generated by Square for the
    `InventoryAdjustment`.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional ID provided by the application to tie the
    `InventoryAdjustment` to an external
    system.
    """

    from_state: typing_extensions.NotRequired[InventoryState]
    """
    The [inventory state](entity:InventoryState) of the related quantity
    of items before the adjustment.
    See [InventoryState](#type-inventorystate) for possible values
    """

    to_state: typing_extensions.NotRequired[InventoryState]
    """
    The [inventory state](entity:InventoryState) of the related quantity
    of items after the adjustment.
    See [InventoryState](#type-inventorystate) for possible values
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Location](entity:Location) where the related
    quantity of items is being tracked.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the
    [CatalogObject](entity:CatalogObject) being tracked.
    """

    catalog_object_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [type](entity:CatalogObjectType) of the [CatalogObject](entity:CatalogObject) being tracked. 
    
    The Inventory API supports setting and reading the `"catalog_object_type": "ITEM_VARIATION"` field value. 
    In addition, it can also read the `"catalog_object_type": "ITEM"` field value that is set by the Square Restaurants app.
    """

    quantity: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The number of items affected by the adjustment as a decimal string.
    Can support up to 5 digits after the decimal point.
    """

    total_price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total price paid for goods associated with the
    adjustment. Present if and only if `to_state` is `SOLD`. Always
    non-negative.
    """

    occurred_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A client-generated RFC 3339-formatted timestamp that indicates when
    the inventory adjustment took place. For inventory adjustment updates, the `occurred_at`
    timestamp cannot be older than 24 hours or in the future relative to the
    time of the request.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    An RFC 3339-formatted timestamp that indicates when the inventory adjustment is received.
    """

    source: typing_extensions.NotRequired[SourceApplicationParams]
    """
    Information about the application that caused the
    inventory adjustment.
    """

    employee_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Employee](entity:Employee) responsible for the
    inventory adjustment.
    """

    team_member_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Team Member](entity:TeamMember) responsible for the
    inventory adjustment.
    """

    transaction_id: typing_extensions.NotRequired[str]
    """
    The Square-generated ID of the [Transaction](entity:Transaction) that
    caused the adjustment. Only relevant for payment-related state
    transitions.
    """

    refund_id: typing_extensions.NotRequired[str]
    """
    The Square-generated ID of the [Refund](entity:Refund) that
    caused the adjustment. Only relevant for refund-related state
    transitions.
    """

    purchase_order_id: typing_extensions.NotRequired[str]
    """
    The Square-generated ID of the purchase order that caused the
    adjustment. Only relevant for state transitions from the Square for Retail
    app.
    """

    goods_receipt_id: typing_extensions.NotRequired[str]
    """
    The Square-generated ID of the goods receipt that caused the
    adjustment. Only relevant for state transitions from the Square for Retail
    app.
    """

    adjustment_group: typing_extensions.NotRequired[InventoryAdjustmentGroupParams]
    """
    An adjustment group bundling the related adjustments of item variations through stock conversions in a single inventory event.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawV1TransactionsClient
from ..types.sort_order import SortOrder
from ..core.request_options import RequestOptions
from ..types.v1order import V1Order
from ..types.v1update_order_request_action import V1UpdateOrderRequestAction
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawV1TransactionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1TransactionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1TransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1TransactionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1TransactionsClient
        """
        return self._raw_client

    def v1list_orders(
        self,
        location_id: str,
        *,
        order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        batch_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[V1Order]:
        """
        Provides summary information for a merchant's online store orders.

        Parameters
        ----------
        location_id : str
            The ID of the location to list online store orders for.

        order : typing.Optional[SortOrder]
            The order in which payments are listed in the response.

        limit : typing.Optional[int]
            The maximum number of payments to return in a single response. This value cannot exceed 200.

        batch_token : typing.Optional[str]
            A pagination cursor to retrieve the next set of results for your
            original query to the endpoint.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[V1Order]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.v1transactions.v1list_orders(
            location_id="location_id",
        )
        """
        response = self._raw_client.v1list_orders(
            location_id, order=order, limit=limit, batch_token=batch_token, request_options=request_options
        )
        return response.data

    def v1retrieve_order(
        self, location_id: str, order_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> V1Order:
        """
        Provides comprehensive information for a single online store order, including the order's history.

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        V1Order
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.v1transactions.v1retrieve_order(
            location_id="location_id",
            order_id="order_id",
        )
        """
        response = self._raw_client.v1retrieve_order(location_id, order_id, request_options=request_options)
        return response.data

    def v1update_order(
        self,
        location_id: str,
        order_id: str,
        *,
        action: V1UpdateOrderRequestAction,
        shipped_tracking_number: typing.Optional[str] = OMIT,
        completed_note: typing.Optional[str] = OMIT,
        refunded_note: typing.Optional[str] = OMIT,
        canceled_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> V1Order:
        """
        Updates the details of an online store order. Every update you perform on an order corresponds to one of three actions:

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        action : V1UpdateOrderRequestAction
            The action to perform on the order (COMPLETE, CANCEL, or REFUND).
            See [V1UpdateOrderRequestAction](#type-v1updateorderrequestaction) for possible values

        shipped_tracking_number : typing.Optional[str]
            The tracking number of the shipment associated with the order. Only valid if action is COMPLETE.

        completed_note : typing.Optional[str]
            A merchant-specified note about the completion of the order. Only valid if action is COMPLETE.

        refunded_note : typing.Optional[str]
            A merchant-specified note about the refunding of the order. Only valid if action is REFUND.

        canceled_note : typing.Optional[str]
            A merchant-specified note about the canceling of the order. Only valid if action is CANCEL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        V1Order
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.v1transactions.v1update_order(
            location_id="location_id",
            order_id="order_id",
            action="COMPLETE",
        )
        """
        response = self._raw_client.v1update_order(
            location_id,
            order_id,
            action=action,
            shipped_tracking_number=shipped_tracking_number,
            completed_note=completed_note,
            refunded_note=refunded_note,
            canceled_note=canceled_note,
            request_options=request_options,
        )
        return response.data


class AsyncV1TransactionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1TransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1TransactionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1TransactionsClient
        """
        return self._raw_client

    async def v1list_orders(
        self,
        location_id: str,
        *,
        order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        batch_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[V1Order]:
        """
        Provides summary information for a merchant's online store orders.

        Parameters
        ----------
        location_id : str
            The ID of the location to list online store orders for.

        order : typing.Optional[SortOrder]
            The order in which payments are listed in the response.

        limit : typing.Optional[int]
            The maximum number of payments to return in a single response. This value cannot exceed 200.

        batch_token : typing.Optional[str]
            A pagination cursor to retrieve the next set of results for your
            original query to the endpoint.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[V1Order]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.v1transactions.v1list_orders(
                location_id="location_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.v1list_orders(
            location_id, order=order, limit=limit, batch_token=batch_token, request_options=request_options
        )
        return response.data

    async def v1retrieve_order(
        self, location_id: str, order_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> V1Order:
        """
        Provides comprehensive information for a single online store order, including the order's history.

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        V1Order
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.v1transactions.v1retrieve_order(
                location_id="location_id",
                order_id="order_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.v1retrieve_order(location_id, order_id, request_options=request_options)
        return response.data

    async def v1update_order(
        self,
        location_id: str,
        order_id: str,
        *,
        action: V1UpdateOrderRequestAction,
        shipped_tracking_number: typing.Optional[str] = OMIT,
        completed_note: typing.Optional[str] = OMIT,
        refunded_note: typing.Optional[str] = OMIT,
        canceled_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> V1Order:
        """
        Updates the details of an online store order. Every update you perform on an order corresponds to one of three actions:

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        action : V1UpdateOrderRequestAction
            The action to perform on the order (COMPLETE, CANCEL, or REFUND).
            See [V1UpdateOrderRequestAction](#type-v1updateorderrequestaction) for possible values

        shipped_tracking_number : typing.Optional[str]
            The tracking number of the shipment associated with the order. Only valid if action is COMPLETE.

        completed_note : typing.Optional[str]
            A merchant-specified note about the completion of the order. Only valid if action is COMPLETE.

        refunded_note : typing.Optional[str]
            A merchant-specified note about the refunding of the order. Only valid if action is REFUND.

        canceled_note : typing.Optional[str]
            A merchant-specified note about the canceling of the order. Only valid if action is CANCEL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        V1Order
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.v1transactions.v1update_order(
                location_id="location_id",
                order_id="order_id",
                action="COMPLETE",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.v1update_order(
            location_id,
            order_id,
            action=action,
            shipped_tracking_number=shipped_tracking_number,
            completed_note=completed_note,
            refunded_note=refunded_note,
            canceled_note=canceled_note,
            request_options=request_options,
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .dispute_evidence_file import DisputeEvidenceFile
from .dispute_evidence_type import DisputeEvidenceType
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DisputeEvidence(UncheckedBaseModel):
    evidence_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-generated ID of the evidence.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-generated ID of the evidence.
    """

    dispute_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the dispute the evidence is associated with.
    """

    evidence_file: typing.Optional[DisputeEvidenceFile] = pydantic.Field(default=None)
    """
    Image, PDF, TXT
    """

    evidence_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Raw text
    """

    uploaded_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the evidence was uploaded, in RFC 3339 format.
    """

    evidence_type: typing.Optional[DisputeEvidenceType] = pydantic.Field(default=None)
    """
    The type of the evidence.
    See [DisputeEvidenceType](#type-disputeevidencetype) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyProgramAccrualRuleCategoryDataParams(typing_extensions.TypedDict):
    """
    Represents additional data for rules with the `CATEGORY` accrual type.
    """

    category_id: str
    """
    The ID of the `CATEGORY` [catalog object](entity:CatalogObject) that buyers can purchase to earn
    points.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .catalog_item_food_and_beverage_details_dietary_preference import (
    CatalogItemFoodAndBeverageDetailsDietaryPreferenceParams,
)
from .catalog_item_food_and_beverage_details_ingredient import CatalogItemFoodAndBeverageDetailsIngredientParams


class CatalogItemFoodAndBeverageDetailsParams(typing_extensions.TypedDict):
    """
    The food and beverage-specific details of a `FOOD_AND_BEV` item.
    """

    calorie_count: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The calorie count (in the unit of kcal) for the `FOOD_AND_BEV` type of items.
    """

    dietary_preferences: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[CatalogItemFoodAndBeverageDetailsDietaryPreferenceParams]]
    ]
    """
    The dietary preferences for the `FOOD_AND_BEV` item.
    """

    ingredients: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[CatalogItemFoodAndBeverageDetailsIngredientParams]]
    ]
    """
    The ingredients for the `FOOD_AND_BEV` type item.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .job_assignment_pay_type import JobAssignmentPayType
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class JobAssignment(UncheckedBaseModel):
    """
    Represents a job assigned to a [team member](entity:TeamMember), including the compensation the team
    member earns for the job. Job assignments are listed in the team member's [wage setting](entity:WageSetting).
    """

    job_title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the job.
    """

    pay_type: JobAssignmentPayType = pydantic.Field()
    """
    The current pay type for the job assignment used to
    calculate the pay amount in a pay period.
    See [JobAssignmentPayType](#type-jobassignmentpaytype) for possible values
    """

    hourly_rate: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The hourly pay rate of the job. For `SALARY` pay types, Square calculates the hourly rate based on
    `annual_rate` and `weekly_hours`.
    """

    annual_rate: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The total pay amount for a 12-month period on the job. Set if the job `PayType` is `SALARY`.
    """

    weekly_hours: typing.Optional[int] = pydantic.Field(default=None)
    """
    The planned hours per week for the job. Set if the job `PayType` is `SALARY`.
    """

    job_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [job](entity:Job).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing


class LoyaltyEventLocationFilterParams(typing_extensions.TypedDict):
    """
    Filter events by location.
    """

    location_ids: typing.Sequence[str]
    """
    The [location](entity:Location) IDs for loyalty events to query.
    If multiple values are specified, the endpoint uses 
    a logical OR to combine them.
    """
